# -*- coding: utf-8 -*-
# Copyright (C) 2008-2009 Jan Hülsbergen <jan@afoo.de>
#
# This file is part of cmdlib.
#
# cmdlib is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cmdlib is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with cmdlib.  If not, see <http://www.gnu.org/licenses/>.

import sys
import time

from cmdlib.colors import colorize, RED, GREEN

class Init(object):
    r"""base class for implementing init-scripts

    (yes, I really do that in Python)

    >>> from StringIO import StringIO
    >>> s = StringIO()

    >>> class MyService(Init):
    ...     name = 'Service'
    ...     def do_start(self):
    ...         self.out.write(',')
    ...         return True

    >>> i = MyService(s)
    >>> i.start()
    True
    >>> i.stop()
    True
    >>> i.restart()
    True

    >>> s.write('end')
    >>> s.seek(0)
    >>> print s.read()
    Starting Service...,done.
    Stopping Service...done.
    Stopping Service...done.
    Starting Service...,done.
    end

    """

    name = 'some service'

    def __init__(self, out=sys.stdout):
        self.out = out
        self.cmds = dict(
            start=self.start,
            stop=self.stop,
            restart=self.restart)

    def usage(self):
        print >>sys.stderr, 'Usage:', sys.argv[0], '|'.join(self.cmds)
        sys.exit(1)

    def main(self):
        try:
            cmd = sys.argv[1]
        except IndexError:
            self.usage()
        self.cmds.get(cmd, self.usage)()

    def start(self):
        self.log('Starting %(name)s...')
        ret = getattr(self, 'do_start', self._noop)()
        if ret:
            self.log_success()
            return True
        self.log_failure()
        return False

    def stop(self):
        self.log('Stopping %(name)s...')
        ret = getattr(self, 'do_stop', self._noop)()
        if ret:
            self.log_success()
            return True
        self.log_failure()
        return False

    def restart(self):
        action = getattr(self, 'do_restart', None)
        if action is None:
            if not self.stop():
                return False
            time.sleep(2)
            return self.start()
        self.log('Restarting %(name)s...')
        if action():
            self.log_success()
            return True
        self.log_failure()
        return False

    def log(self, msg):
        msg = msg % dict(name=self.name)
        self.out.write(msg)
        self.out.flush()

    def log_success(self):
        msg = self._color_if_tty(GREEN, 'done.')
        self.log(msg + '\n')

    def log_failure(self):
        msg = self._color_if_tty(RED, 'FAIL!')
        self.log(msg + '\n')

    def _color_if_tty(self, color, msg):
        if self.out.isatty():
            return colorize(color, msg)
        return msg

    def _noop(self):
        return True


if __name__ == '__main__':
    import doctest
    doctest.testmod()
