# -*- coding: utf-8 -*-
# Copyright (C) 2008-2009 Jan Hülsbergen <jan@afoo.de>
#
# This file is part of cmdlib.
#
# cmdlib is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cmdlib is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with cmdlib.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import tempfile


class TransformerDoneError(StandardError):
    pass


class Transformer(object):
    r'''Simple Transformer to do sed-like thinks to a file.

    Usage:
    >>> import tempfile, os, re
    >>> f, n = tempfile.mkstemp()
    >>> f = os.fdopen(f, "w")
    >>> f.write("foo\nbar\nBaz\nspam!!!")
    >>> f.close()

    >>> t = Transformer(n)
    >>> t.add_trans("!", "?")
    >>> t.add_re_trans(re.compile(r"b\w", re.I), "")
    >>> t.run()

    >>> f = open(n)
    >>> print f.read()
    foo
    r
    z
    spam???

    '''

    def __init__(self, file_name):
        self.file_name = file_name
        self.fh = open(file_name)
        self.rtrans = list()
        self.strans = list()
        self.ran = False

    def add_trans(self, old, new):
        '''replace any occurence of string/regexp old with new'''
        self.strans.append((old, new))

    def add_re_trans(self, old, new):
        '''same as add_trans, but with regexps'''
        if isinstance(old, basestring):
            old = re.compile(old)
        self.rtrans.append((old, new))

    def run(self):
        if self.ran:
            raise TransformerDoneError('this transformer already ran')
        tmpfd, tmpname = tempfile.mkstemp()
        tmpfh = os.fdopen(tmpfd, 'w')
        for line in self.fh:
            line = self._transform(line)
            tmpfh.write(line)
        self.fh.close()
        tmpfh.flush()
        tmpfh.close()
        os.rename(tmpname, self.file_name)
        self.ran = True

    def _transform(self, s):
        for old, new in self.rtrans:
            s = old.sub(new, s)
        for old, new in self.strans:
            s = s.replace(old, new)
        return s


def simple_trans(file_name, old, new):
    """simple translation"""
    t = Transformer(file_name)
    t.add_trans(old, new)
    t.run()


def simple_re_trans(file_name, old, new):
    """simple translation using regexp"""
    t = Transformer(file_name)
    t.add_re_trans(old, new)
    t.run()


if __name__ == '__main__':
    import doctest
    doctest.testmod()
